import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * The agent will explore the wumpus world in search of the gold. They have at their displosal
 * a single arrow to kill the wumpus. They can also detect using 5 senses. This being:
 *    1) Detecting a glitter: The gold will glitter
 *    2) Detecting a breeze: Breezes appear in the directly adjacent blocks to a pit.
 *    3) Detecting a stench: The wumpus will put off a stench into the adjacent squares.
 *    4) Detecting a bump: Bumps occur when the agent attempts to walk beyond the confines of
 *       the world.
 *    5) Detecting a scream: If the agent fires his/her arrow forward into the wumpus, this will
 *       effectively slay the beast, causing it to let out a piercing cry.
 *       
 *    The agent has at his disposal these methods in order to determine their next move of action:
 *      *From the agent*
 *         *turnLeft(), turnRight() - turns the player in the direction indicated.
 *         *move() - advances the player one grid space forward in the direction they are currently
 *                    facing. This direction can also be checked within the private String heading,
 *                    which will indicate NORTH, EAST, WEST, or SOUTH.
 *         *senseBreeze(), senseStench(), senseGlitter() - these methods will return true if the
 *                    player is located on a space with the breeze, stench, or gold respectively.
 *         *shoot() - fires an arrow along the players current heading. If the wumpus is along said
 *                     heading, this method returns true to indicate a dead wumpus. UI will update 
 *                     display a scream heard. This decrements the arrow count if available, or
 *                     returns false if out of arrows. Remaining arrows are found in arrowsLeft.
 * 
 * @author Christopher McKee
 * @version W13
 */
public class Agent extends Actor
{
    private GreenfootImage foreground, eastImage, southImage, westImage, northImage;
    private TheWumpusWorld world;
    private String heading;
    private int arrowsLeft;
    public boolean isAlive;
    
    //Custom variables for the player to use while navigating
    private int[][] tracking;
    private boolean bumped, turning;   
    /* This method will be written by the student to include the logic needed by the agent.
     * As of now, it is has a very simple ai that is being used to show the use of the various
     * agent methods.
     */
    private void makeNextMove()
    {
        //senseGlitter detects if you are on the gold
        if(this.senseGlitter() == true)
        {
            world.takeGold();
        }
        //senseStench detects if you are currently adjacent to the wumpus
        else if(this.senseStench() == true)
        {
            //shoot fires the arrow along your current heading, returning true on a hit
            if(this.shoot() == false)
                this.turnLeft();
        }
        //senseBreeze detects if you are adjacent to a pit
        //bumped indicates if you previously failed to move forward
        //turning indicates if you preciously rotated your agent
        else if((this.senseBreeze() == true || this.bumped == true) && this.turning == false)
        {
            bumped = false;
            if(Greenfoot.getRandomNumber(1000) < 500)
                this.turnLeft();
            else
                this.turnRight();
                
            turning = true;
        }
        else
        {
            //player moves forward one space, if this 
            if(this.move() == false)
                bumped = true;
            else
                bumped = false;
        
            turning = false;
        }
    }
    
    /**************************************************************************************/
    
    
    public Agent(TheWumpusWorld playSpace)
    {
        int size;
        world = playSpace;
        tracking = new int[world.getPlayWidth()][world.getPlayWidth()];
        eastImage = new GreenfootImage("AgentEast.png");
        southImage = new GreenfootImage("AgentSouth.png");
        westImage = new GreenfootImage("AgentWest.png");
        northImage = new GreenfootImage("AgentNorth.png");
        size = world.getCellSize() * 9 / 10;
        eastImage.scale(size, size);
        southImage.scale(size, size);
        westImage.scale(size, size);
        northImage.scale(size, size);
        heading = "EAST";
        foreground = eastImage;
        setImage(foreground);
        isAlive = true;
        
        //Initialize your custom variables
        bumped = turning = false;
    }
    /**
     * Act - do whatever the Agent wants to do. This method is called whenever
     * the 'Act' or 'Run' button gets pressed in the environment.
     */
    public void act() 
    {
        if(isAlive == true)
            makeNextMove();
            
        world.score.updateImage();
    }
    
    private void turnLeft()
    {
        world.bump.setScore(0);
        if(heading.equals("NORTH"))
        {
            heading = "WEST";
            foreground = westImage;
        }
        else if(heading.equals("EAST"))
        {
            heading = "NORTH";
            foreground = northImage;
        }
        else if(heading.equals("SOUTH"))
        {
            heading = "EAST";
            foreground = eastImage;
        }
        else if(heading.equals("WEST"))
        {
            heading = "SOUTH";
            foreground = southImage;
        }
        setImage(foreground);
        world.score.subtract(10);
    }
    
    private void turnRight()
    {
        world.bump.setScore(0);
        if(heading.equals("NORTH"))
        {
            heading = "EAST";
            foreground = eastImage;
        }
        else if(heading.equals("EAST"))
        {
            heading = "SOUTH";
            foreground = southImage;
        }
        else if(heading.equals("SOUTH"))
        {
            heading = "WEST";
            foreground = westImage;
        }
        else if(heading.equals("WEST"))
        {
            heading = "NORTH";
            foreground = northImage;
        }
        setImage(foreground);
        world.score.subtract(10);
    }
    
    private boolean move()
    {
        world.bump.setScore(0);
        if(world.wallToThe(heading))
        {
            world.bump.setScore(1);
            world.score.subtract(20);
            return false;
        }
        else
        {
            if(heading.equals("NORTH"))
                setLocation(getX(), getY() - 1);
            else if(heading.equals("EAST"))
                setLocation(getX() + 1, getY());
            else if(heading.equals("SOUTH"))
                setLocation(getX(), getY() + 1);
            else if(heading.equals("WEST"))
                setLocation(getX() - 1, getY());
        }
        
        if(world.testForDeath())
            world.score.subtract(1000);
        else
            world.score.subtract(20);
            
        return true;
    }
    
    private boolean senseBreeze()
    {
         return world.testForBreeze();
    }
    
    private boolean senseStench()
    {
        return world.testForStench();
    }
    
    private boolean senseGlitter() 
    {
        return world.testForGlitter();
    }
    
    private boolean shoot()
    {
        world.bump.setScore(0);
        if(arrowsLeft > 0)
        {
            Arrow shot = new Arrow(world, heading);
            world.addObject(shot, getX(), getY());
            world.arrows.setScore(--arrowsLeft);
            world.score.subtract(20);
            if(world.checkTrajectory(heading))
            {
                world.killWumpus();
                world.scream.setScore(1);
                return true;
            }
        }
        world.scream.setScore(0);
        return false;
    }
    
    public void setArrowCount(int count)
    {
        this.arrowsLeft = count;    
    }
}
